#!/bin/bash
# (c) wiimm (at) wiimm.de -- 2014-01-03

#
#------------------------------------------------------------------------------
# setup

printf "\nSetup...\n"

script="${0##*/}"
workdir=./patch-dir
srclist="*.iso *.ciso *.wdf *.wia *.wbfs"

CONFIG="./config.def"
CFG_LANGUAGE="$LC_MESSAGE"
[[ $CFG_LANGUAGE = "" ]] && CFG_LANGUAGE="${LC_CTYPE%%_*}"
[[ $CFG_LANGUAGE = "" ]] && CFG_LANGUAGE=en

CFG_MSGLANG="$CFG_LANGUAGE"
MSGLANG_LIST="en en+ de de+"

ISOMODE_LIST="iso ciso wdf wbfs wia"

[[ $VERBOSE = 1 ]] || VERBOSE=0
[[ $AUTORUN = 1 ]] || AUTORUN=0
for p in "$@"
do
    [[ $p == -v || $p = --verbose ]] && VERBOSE=1
    [[ $p == -a || $p = --autorun ]] && AUTORUN=1
    [[ $p == -h || $p = --help ]] && AUTORUN=1
done

#
#------------------------------------------------------------------------------
# system and bin path

#--- BASEDIR

if [[ ${BASH_SOURCE:0:1} == / ]]
then
    BASEDIR="${BASH_SOURCE%/*}"
    [[ $BASEDIR = "" ]] && BASEDIR=/
else
    BASEDIR="$PWD/$BASH_SOURCE"
    BASEDIR="${BASEDIR%/*}"
fi
#echo "0=$0"
#echo "BASH_ARGV=${BASH_ARGV[*]}"
((VERBOSE)) && echo "BASH_SOURCE = ${BASH_SOURCE[*]}"
((VERBOSE)) && echo "BASEDIR     = $BASEDIR"

#--- predefine

ORIGPATH="$PATH"
BINDIR="$BASEDIR/cygwin"
((VERBOSE)) && echo "BINDIR      = $BINDIR"
if [[ -d $BINDIR ]]
then
    chmod u+x "$BINDIR"/* 2>/dev/null
    export PATH="$BINDIR:$ORIGPATH"
fi

#--- find system

SYSTEM="$( uname -s | tr '[A-Z]' '[a-z]' )"
MACHINE="$( uname -m | tr '[A-Z]' '[a-z]' )"
((VERBOSE)) && echo "SYSTEM      = $SYSTEM"
((VERBOSE)) && echo "MACHINE     = $MACHINE"

case "$SYSTEM-$MACHINE" in
    darwin-*)		HOST=mac ;;
    linux-x86_64)	HOST=linux64 ;;
    linux-*)		HOST=linux32 ;;
    cygwin*)		HOST=cygwin ;;
    *)			HOST=- ;;
esac

#--- setup BINDIR and PATH

BINDIR="$BASEDIR/$HOST"
((VERBOSE)) && echo "BINDIR      = $BINDIR"
if [[ -d $BINDIR ]]
then
    chmod u+x "$BINDIR"/* 2>/dev/null
    export PATH="$BINDIR:$ORIGPATH"
fi

((VERBOSE)) && echo "AUTORUN     = $AUTORUN"

#
#------------------------------------------------------------------------------
# check tools

needed_tools="awk bash cat chmod cp cut diff find ln mkdir mv rm
	sed sort tar touch tr uname uniq unzip wc which
	wit wszst wbmgt wimgt wstrt"

err=

for tool in $needed_tools
do
    if ! which $tool >/dev/null 2>&1
    then
	err="$err $tool"
    fi
done

if [[ $err != "" ]]
then
    echo "*** Missing tools:$err" >&2
    exit 1
fi

#
#------------------------------------------------------------------------------
# function printlog()

function printlog()
{
    ((quiet)) && return 0
    local msg="$(printf "$@")"
    local sep="#######################################"
    sep="#$sep$sep"
    local len len1 len2
    let len=${#msg}
    let len1=(57-len)/2
    let len2=57-len1
    printf "\n%s\n########## %*s%-*s ##########\n%s\n\n" \
	"$sep" $len1 "" $len2 "$msg" "$sep"
}

#
#------------------------------------------------------------------------------
# configuration

for d in "$BASEDIR" "$BASEDIR/.." .
do
    LANG_SRC="$d/locale"
    if [[ -d $LANG_SRC ]]
    then
	LANG_LIST="$( cd "$LANG_SRC"; echo *.sh | sed 's/.sh//g' )"
	[[ $LANG_LIST == "*" ]] || break
    fi
done
((VERBOSE)) && echo "LANG_SRC    = $LANG_SRC"
((VERBOSE)) && echo "LANG_LIST   = $LANG_LIST"

if [[ -s $CONFIG ]]
then
    eval $(sed 's/^/CFG_/' "$CONFIG")
fi

function save_config()
{
    cat <<- --EOT-- >"$CONFIG"
	LANGUAGE="$CFG_LANGUAGE"
	MSGLANG="$CFG_MSGLANG"
	ISOMODE="$CFG_ISOMODE"
	SPLITISO="$CFG_SPLITISO"
	PRIV_SAVEGAME="$CFG_PRIV_SAVEGAME"
	--EOT--
}

function print_config()
{
    local format=" %-30s : %s\n"
    printf "\n---------------------------------------\n"
    printf "$format" "$T_LANGUAGE" "$CFG_LANGUAGE"
    printf "$format" "$T_MSGLANG" "$CFG_MSGLANG"
    printf "$format" "$T_ISOMODE" "$CFG_ISOMODE"
    printf "$format" "$T_SPLITISO" "$CFG_SPLITISO"
    printf "$format" "$T_PRIV_SAVEGAME" "${CFG_PRIV_SAVEGAME#*=}"
    printf -- "---------------------------------------\n\n"
}

function validate_config()
{
    [[ $CFG_LANGUAGE = "" ]]		&& return 1
    [[ $CFG_MSGLANG = "" ]]		&& return 1
    [[ $CFG_ISOMODE = "" ]]		&& return 1
    [[ $CFG_SPLITISO = "" ]]		&& return 1
    [[ $CFG_PRIV_SAVEGAME = "" ]]	&& return 1
    return 0
}

function config_options()
{
    CFG_OPT=("--riiv-mode=$ENABLE_RIIV")
    [[ ${CFG_ISOMODE} != "" ]]		&& CFG_OPT=("${CFG_OPT[@]}" --${CFG_ISOMODE})
    [[ ${CFG_SPLITISO:0:1} = 1 ]]	&& CFG_OPT=("${CFG_OPT[@]}" --split)
    [[ ${CFG_PRIV_SAVEGAME:0:1} = 1 ]]	&& CFG_OPT=("${CFG_OPT[@]}" --savegame)
}

#
#------------------------------------------------------------------------------
# load_locale()

function load_locale()
{
    local src="$LANG_SRC/$1.sh"
    [[ -s $src ]] && . "$src"
}

load_locale de			# first get german texts
load_locale en			# ... then english texts (both fall back)
load_locale "$CFG_LANGUAGE"	# now scan predefined language

#
#------------------------------------------------------------------------------
# read_char()

function read_char()
{
    local prompt="$1"
    local default="$2"
    shift 2
    while true
    do
	local in
	read -n1 -p "$prompt [$default]: " -s in || exit 1
	if [[ $in = "" ]]
	then
	    echo "$default"
	    echo "$default" >&2
	    return 0
	fi

	for (( i=1; i <= $#; i++ ))
	do
	    local p="${!i}"
	    if [[ $in == ${p:0:1} ]]
	    then
		echo "${p:1}"
		echo "${p:1}" >&2
		return 0
	    fi
	done
	echo >&2
    done
}

#
#------------------------------------------------------------------------------
# read_mode()

function read_mode()
{
    local prompt="$1"
    local default="$2"
    shift 2
    while true
    do
	local in
	read -p "$prompt [$default]: " in || exit 1
	[[ $in = "" ]] && in="$default"
	local len=${#in}
	local found=
	local count=0
	for (( i=1; i <= $#; i++ ))
	do
	    local p="${!i}"
	    if [[ $in == $p ]]
	    then
		echo "$p"
		return 0
	    fi
	    if ((len)) && [[ $in == ${p:0:(len)} ]]
	    then
		let count++
		found="$p"
	    fi
	done
	if (( count == 1 ))
	then
	    echo "$found"
	    return 0
	fi
    done
}

#
#------------------------------------------------------------------------------
# read_noyes()

function read_noyes()
{
    local prompt="$1"
    local default="$2"
    [[ ${2:0:1} = 0 ]] && default="$T_NO"
    [[ ${2:0:1} = 1 ]] && default="$T_YES"
    local answer="$(read_mode "$prompt ($T_NO,$T_YES)" "$default" "$T_NO" "$T_YES" 0 1 )"
    [[ $answer = $T_NO  || $answer = 0 ]] && answer="0=$T_NO"
    [[ $answer = $T_YES || $answer = 1 ]] && answer="1=$T_YES"
    echo "$answer"
    return 0
}

#
#------------------------------------------------------------------------------
# ask_config()

function ask_config()
{
    #----- dialog language

    if [[ $AUTORUN != 1 && ${ASK_LANGUAGE:=1} = 1 ]]
    then
	printf "$T_LANGUAGE_INFO\n"
	CFG_LANGUAGE="$(read_mode "* $T_LANGUAGE (${LANG_LIST// /,})" "$CFG_LANGUAGE" $LANG_LIST)"
    fi
    load_locale "$CFG_LANGUAGE"


    #----- game language

    if [[ $AUTORUN != 1 && ${ASK_MSGLANG:=1} = 1 ]]
    then
	printf "$T_MSGLANG_INFO\n"
	CFG_MSGLANG="$(read_mode "* $T_MSGLANG (${MSGLANG_LIST// /,})" "$CFG_MSGLANG" $MSGLANG_LIST)"
    fi
    export CFG_MSGLANG


    #----- iso mode (or Riivolution)

    [[ -d $BASEDIR/riivolution ]] || ENABLE_RIIV=0

    local info="$T_ISOMODE_INFO"
    local list="$ISOMODE_LIST"
    if [[ $ENABLE_RIIV = 1 ]]
    then
	info="${info}${T_ISOMODE_RIIV1}"
	list="$list riiv"
    elif [[ $ENABLE_RIIV = 2 ]]
    then
	info="${info}${T_ISOMODE_RIIV2}"
	list="$list riiv riiv-"
    elif (( ENABLE_RIIV > 2 ))
    then
	info="${info}${T_ISOMODE_RIIV3}"
	list="$list riiv"
    fi

    if [[ $AUTORUN != 1 && ${ASK_ISOMODE:=1} = 1 ]]
    then
	printf "$info\n"
	CFG_ISOMODE="$(read_mode "* $T_ISOMODE (${list// /,})" "$CFG_ISOMODE" $list)"
    fi

    CREATE_RIIV=0
    [[ $CFG_ISOMODE = riiv ]] && CREATE_RIIV=1
    [[ $CFG_ISOMODE = riiv- ]] && CREATE_RIIV=2


    #----- split image

    if [[ $AUTORUN != 1 && ${ASK_SPLITISO:=1} = 1 && $CREATE_RIIV = 0 ]]
    then
	printf "$T_SPLITISO_INFO\n"
	CFG_SPLITISO="$(read_noyes "* $T_SPLITISO" "$CFG_SPLITISO")"
    fi


    #----- alternate save game

    if [[ $AUTORUN != 1 && ${ASK_SAVEGAME:=1} = 1  && $CREATE_RIIV = 0 ]]
    then
	printf "$T_SAVEGAME_INFO\n"
	CFG_PRIV_SAVEGAME="$(read_noyes "* $T_PRIV_SAVEGAME" "$CFG_PRIV_SAVEGAME")"
    fi


    #----- save config

    save_config
}

#
#------------------------------------------------------------------------------
# end

